----- GADGET MAKING WHOLE MAP -----

function gadget:GetInfo()
	return {
		name 	= "notAmap",
		desc 	= "Proceduraly created map",
		author 	= "PepeAmpere",
		date 	= "12th December, 2012",
		license = "BY-NC-SA",
		layer	= 0,
		version = "0.041",
		enabled = true,
	}
end

----- HINTS ------
-- for testing/debug complicated functions just change tileScale => 1 -> 2 or 64 :)
-- use filter["writer"](yourFilterMap) for observing your filter map in logfile

local spSetHeightMapFunc	= Spring.SetHeightMapFunc
local spSetHeightMap		= Spring.SetHeightMap
local spAddHeightMap		= Spring.AddHeightMap 
local spLevelHeightMap		= Spring.LevelHeightMap

floor						= math.floor
random						= math.random
sin							= math.sin
cos							= math.cos

mapOptions					= {}
mapHash						= {}
fullMapHash					= 0
heightMap					= {}
mapKnowledge				= {}

choosenMapType				= "random"
choosenMapStyle				= "random"

tileScale					= 4
tileSize					= Game.squareSize*tileScale
mapX						= Game.mapSizeX
mapZ						= Game.mapSizeZ
mapXdivs					= floor(mapX / tileSize)
mapZdivs					= floor(mapZ / tileSize)
mapCount					= mapXdivs * mapZdivs

defaultHeightLevel 	= Spring.GetGroundHeight(10,10)

toolNameToType				= {} 

--include "LuaRules/Configs/builders.lua"		-- library of builders (THIS IS LOADED LATER IN INIT)

--- MATH ---
local function MySinRandomizator(hash,members)
	return floor((((sin(hash) + 1)/2) * members) + 1)
end

--- MAPPING INIT ---
-- init of toolNameToType
local function InitToolTypes()
	for k,v in pairs(digger) do
		toolNameToType[k] = "digger" 
	end

	for k,v in pairs(filter) do
		toolNameToType[k] = "filter" 
	end
end
-- init of hashNumber
local function PrepareHashArray(number)
	local current = number
	for i=1,7 do
		mapHash[i] 	= current % 10
		current		= floor(current/10)
		--Spring.Echo(mapHash[i])
	end
	return number
end
-- init of heightmap
local function HeightMapInit(defaultHeight)
	for i=0,mapXdivs-1 do
		for j=0,mapZdivs-1 do
			heightMap[i*mapZdivs+j+1] = defaultHeight
		end
	end
end

local function GetRandomType(typeName,hashNumber)
	local mapTypeList 		= {}
	local mapTypeCounter 	= 0
	if 		(typeName == "randomland") then	-- ?! randomLand -> randomland (startscript lowercasing)
		-- just land maps
		for k,v in pairs(mapType) do
			if (not v.seaMap) then
				mapTypeCounter 				= mapTypeCounter + 1
				mapTypeList[mapTypeCounter] = k 
			end
		end
	elseif 	(typeName == "randomsea") then 	-- ?! randomSea -> randomsea (startscript lowercasing)
		-- just sea maps
		for k,v in pairs(mapType) do
			if (v.seaMap) then
				mapTypeCounter 				= mapTypeCounter + 1
				mapTypeList[mapTypeCounter] = k 
			end
		end
	else
		-- general random
		for k,v in pairs(mapType) do
			mapTypeCounter 				= mapTypeCounter + 1
			mapTypeList[mapTypeCounter] = k 
		end
	end
	local choosenIndex = MySinRandomizator(hashNumber,mapTypeCounter)
	if (choosenIndex >= 1 and choosenIndex <= mapTypeCounter) then
	
		--- TEMPORARY CONDTION UNTIL ALL MAPTYPES FINISHED ---
		local finishedTypes = {
			"craterlake",
			"desert",
			"mediterranean",
			"mountains",
			"onecontinent",
		}
		local choosenFinished = false
		for i=1,#finishedTypes do
			if (mapTypeList[choosenIndex] == finishedTypes[i]) then
				choosenFinished = true
				return mapTypeList[choosenIndex]
			end
		end
		--- END OF TEMPORARY CONDITION
		
		if (choosenFinished == false) then
			return "mountains"
		end
	else
		Spring.Echo("notAmap: randomization need enhance")
		return "mediterranean"
	end
end

local function GetRandomStyle(typeName,hashNumber)
	local mapStylesList		= mapType[typeName].styles
	local mapStylesCount 	= #mapStylesList
	local choosenIndex 		= MySinRandomizator(hashNumber,mapStylesCount)
	
	if (choosenIndex >= 1 and choosenIndex <= mapStylesCount) then
		return mapStylesList[choosenIndex]
	else
		Spring.Echo("notAmap: randomization need enhance")
		return mapStylesList[1]
	end

end

--- END OF MAPPING INIT ---

function MainMapEditor(typeName)
	local thisMapType 		= mapType[typeName]
	local thisPlan		 	= thisMapType.plan
	for i=1,#thisPlan do
		local thisBuilder = builder[thisPlan[i]]
		if ((thisBuilder ~= nil) and (#thisBuilder >= 1)) then
			local lastMap  = filter["fullNotActiveMap"](mapXdivs,mapZdivs)
		    for j=1,#thisBuilder do
				local thisAction 	= thisBuilder[j]
				local currentMap
				
				-- NEXT ROUND MAP SELECTION --
				if (thisAction.recentMap) then
					currentMap = lastMap
				else
					currentMap = filter["fullNotActiveMap"](mapXdivs,mapZdivs)
				end
				
				-- EXECUTORS --
				if (toolNameToType[thisAction.name] == "filter") then
					-- FILTER EXECUTING --
					lastMap = filter[thisAction.name](currentMap,unpack(thisAction.params))
				else
					-- DIGGER EXECUTING --
					lastMap = digger[thisAction.name](currentMap,unpack(thisAction.params))
				end
				
				-- DEBUG --
				--Spring.Echo("--")
				--filter["mapWriter"](lastMap)
			end
		end
	end
end


----- SYNCED ------------------------------------------
if (gadgetHandler:IsSyncedCode()) then
-------------------------------------------------------

function gadget:Initialize()
	Spring.Echo("-- NOTaMAP LOG --")
	
	--- READ MAP OPTIONS ---
	mapOptions 	= Spring.GetMapOptions()
	fullMapHash = PrepareHashArray(mapOptions.hash)
	Spring.Echo("notAmap hash: " .. fullMapHash)
	
	--- INCLUDE CONFIG FILES and RECONFING LIBS ---
	include "LuaRules/Configs/filters.lua"			-- library of filters 
	include "LuaRules/Configs/procEls.lua"			-- library of proceduraly created landscape elements
	include "LuaRules/Configs/maskEls.lua"			-- library of mask-created landscape elements
	include "LuaRules/Configs/masks.lua"			-- library of masks (= not full map filters)
	include "LuaRules/Configs/diggers.lua"			-- library of diggers
	include "LuaRules/Configs/mapType.lua"			-- library of map-types
	include "LuaRules/Configs/terraMath.lua"		-- library of some misc functions
	include "LuaRules/Configs/builders.lua"			-- library of builders
	InitToolTypes()	
	
	--- CHOOSE STYLE/TYPE + PLAN + ETC ---
	
	--- START POS ---
	
	--- HEIGHTMAP + METALMAP CREATION ---
	
	-- init --
	local defaultLevel	= defaultHeightLevel
	choosenMapType 		= tostring(mapOptions.maptype)
	HeightMapInit(defaultLevel)
	-- endinit --
	
	-- if random, randomLand or random_sea - get the TypeName
	-- ? why "randomland" instead of "randomLand" and "randomsea" instead of "randomSea"? automatic lowercasing of options done via lobby or spring
	if (choosenMapType == "random" or choosenMapType == "randomland" or choosenMapType == "randomsea") then
		choosenMapType = GetRandomType(choosenMapType,fullMapHash)
	end	
	
	Spring.Echo("notAmap type: " .. choosenMapType)
	MainMapEditor(choosenMapType)	
	
	-- ApplyChanges()
	-- Spring.LevelSmoothMesh(1700,1700,2200,2200,-100)
	
	--- STYLE ---
	choosenMapStyle = GetRandomStyle(choosenMapType,fullMapHash)
	Spring.Echo("notAmap style: " .. choosenMapStyle)

	--- FEATURES ---		
	
	--- SOME END STUFF ---

	Spring.Echo("notAmap creator: finished!")
	
	--- CLEAN ---
    --gadgetHandler:RemoveGadget()
	map = nil
	--Spring.Echo(tostring(collectgarbage(count)))
	--collectgarbage(collect)
	
	Spring.Echo("-- NOTaMAP LOG END --")
end


--------------------------------------------------------
else
----- UNSYNCED -----------------------------------------

end
