----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
-- File:        MapOptions.lua
-- Description: Custom MapOptions file that makes possible to set up variable options before game starts, like ModOptions.lua
-- Author:      SirArtturi, Lurker, Smoth, jK
----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
--	NOTES:
--	- using an enumerated table lets you specify the options order
--
--	These keywords must be lowercase for LuaParser to read them.
--
--	key:		the string used in the script.txt
--	name:		the displayed name
--	desc:		the description (could be used as a tooltip)
--	type:		the option type
--	def:		the default value
--	min:		minimum value for number options
--	max:		maximum value for number options
--	step:		quantization step, aligned to the def value
--	maxlen:	 	the maximum string length for string options
--	items:		array of item strings for list options
--	scope:		'all', 'player', 'team', 'allyteam'			<<< not supported yet >>>
--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

local options = {
--// Sections
	{
		key  = 'map',
		name = 'Map settings',
		desc = 'Choose how the map should be build',
		type = 'section',
	},	
	
	{
		key  = 'Atmosphere',
		name = 'Atmosphere Settings',
		desc = 'Weather and time',
		type = 'section',
	},

	-- {
		-- key  = 'Terrain',
		-- name = 'Terrain Settings',
		-- desc = '',
		-- type = 'section',
	-- },

	-- {
		-- key  = 'Economy',
		-- name = 'Economy Settings',
		-- desc = '',
		-- type = 'section',
	-- },

--// Options
	--// MapType
	{
		key  = "maptype",
		name = "Map Type",
		desc = "Select the map style, that determine the strategic situation on the map.",
		type = "list",
		def  = "random",
		section = 'map',
		items = {
			{ key = "random", 		 	name = "x-Random", 			desc = "Map is choosen randomly, when game starts." },
			{ key = "randomland",   	name = "x-Random Land",   	desc = "Map is choosen randomly, when game starts and players can be sure they start on the same landmass." },
			{ key = "randomsea", 		name = "x-Random Sea", 		desc = "Map is choosen randomly, when game starts and players can be sure theres water between their startpositions" },
			{ key = "atol", 		 	name = "x-Atol",  			desc = "There many corral islands in the circle with ocean around and metal-rich lagoon in the middle." },
			{ key = "battlefield",	 	name = "x-Battlefield",		desc = "In the middle old battlefield with metal, startpositions with small camps around." },
			{ key = "colonisation",	 	name = "x-Colonisaiton",	desc = "Theres continent with players and no resources and second continent with many resources." },
			{ key = "corralsea", 		name = "x-Corral Sea",  	desc = "Clusters of isles in big ocean." },
			{ key = "channel", 		 	name = "x-Channel",  		desc = "Narrow sea channel with cliffs on both sides." },
			{ key = "clanwars",  		name = "x-Clan Wars",		desc = "One long thin isle spliting the map into two parts, players start positions in line on this isle, some small well metal resourced isles around (5-7)." },
			{ key = "craterlake", 		name = "Crater Lake",  		desc = "There is small lake in the middle, a lot of resources around it, rest is not much valuable." },
			{ key = "delta", 		 	name = "x-Delta",  			desc = "Many small rivers spliting the map on many isles." },
			{ key = "desert", 		 	name = "Desert",  			desc = "Flat ground map with low resources and few rock platforms." },
			{ key = "forrest",	 		name = "x-Forrest",			desc = "Hills, lakes and many trees there." },
			{ key = "jungle",	 		name = "x-Jungle",			desc = "Small table or normal mountains around the coast of the map, more in middle, res ress and less strategic places." },
			{ key = "lakes", 		 	name = "x-Lakes",  			desc = "5-8 medium lakes with metal sources, metal-poor flat terrian around." },
			{ key = "mediterranean", 	name = "Mediterranean",  	desc = "Big inland sea with 1-3 small isles in the middle, players start positions around the map." },
			{ key = "mountains", 		name = "Mountains",  		desc = "Like Mediterranean, the flat land around stays, but in the middle there mountains with lot of metal sources." },
			{ key = "moon", 		 	name = "x-Moon",  			desc = "Few big and many small craters." },
			{ key = "oceanpowers", 		name = "x-Ocean Powers",  	desc = "Each player have small continent and theres big ocean around." },
			{ key = "oldworld", 		name = "x-Old World",  		desc = "Some hills, rivers, flat lands, lakes,... combination of all Earth-type TYPES." },
			{ key = "onecontinent", 	name = "x-One Continent",  	desc = "Theres a big continent in them middle sourrounded by ocean, players starts at the coast (inverted Mediterranean, but no lakes in the middle)." },
			{ key = "oceania", 		 	name = "x-Oceania",  		desc = "A lot of small isles, big ocean." },
			{ key = "tablemountain", 	name = "x-TableMountain",  	desc = "Like Mountains , but with one table mountain/platform in the middle." },
			{ key = "globalwar", 		name = "x-Global War", 		desc = "Each team have big continent sourounded by ocean." },
			{ key = "terranova", 		name = "x-Terra_Nova",  	desc = "Like Colonisation, but each team starts from own isle/continent on different place at border of the map." },
			{ key = "tropicalsea", 		name = "x-Tropical Sea",  	desc = "Bigger isles for starting positions with some hill there and many small isles around." },
		},
	},
	
	{
		key  = "mapstyle",
		name = "Map Style",
		desc = "Select the map style, that determine the visual theme of map.",
		type = "list",
		def  = "random",
		section = 'map',
		items = {
			{ key = "random", 		 	name = "x-Random", 			desc = "Map style is choosen randomly, when game starts from list of possible styles." },
		},
	},
	
	{
		key    = "hash",
		name   = "Map hash",
		desc   = "Number determination random procedures when map is generated.",
		type    = 'number',
		def     = 321458,
		min     = 1,
		max     = 999999,
		step    = 1,
		section="map",
    },

	--// Atmosphere
	-- {
		-- key  = "timeofday",
		-- name = "Time of day",
		-- desc = "Night or day?",
		-- type = "list",
		-- def  = "day",
		-- section = 'Atmosphere',
		-- items = {
			-- { key = "dawn",  name = "Dawn",  desc = "Day Time" },
			-- { key = "day",   name = "Day",   desc = "Day Time" },
			-- { key = "night", name = "Night", desc = "Night Time" }
		-- },
	-- },
	
	-- {
		-- key  = 'fog',
		-- name = 'Fog',
		-- desc = "covers the map under a gray mist (can be slow on older hardware)",
		-- type = 'bool',
		-- section = 'Atmosphere',
		-- def  = true,
	-- },
	
	{
		key  = "weather",
		name = "Weather conditions",
		desc = "snowing",
		type = "list",
		def  = "clear",
		section = 'Atmosphere',
		items = {
			{ key = "clear", name = "Clear", desc = "no weather" },
			{ key = "snow", name = "Snowing", desc = "snowy day" }
		},
	},

	--// Terrain
	-- {
		-- key = 'inv',
		-- name = 'Inverted Heightmap',
		-- desc = "flip world",
		-- type = 'bool',
		-- section = 'Terrain',
		-- def = false,
	-- },

	--// Economy
	-- {
		-- key  = 'metal',
		-- name = 'Metal Production',
		-- desc = 'Metal production levels - How much metal is produced per second',
		-- type = 'list',
		-- section = 'Economy',
		-- def  = 'normal',
		-- items	= {
			-- { key = 'low', name = "Low", desc = "Low metal density - for smaller teams" },
			-- { key = 'normal', name = "Normal", desc = "Default metal density" },
			-- { key = 'high', name = "High", desc = "High metal density - for big teams" },
		-- },
	-- },

	-- {
		-- key   = 'wind',
		-- name  = 'Wind Speed',
		-- desc  = 'How strong wind will blow',
		-- type  = 'list',
		-- section = 'Economy',
		-- def   = 'normal',
		-- items = {
			-- { key = 'low', name = "Low", desc = "Weak wind speed - for smaller teams" },
			-- { key = 'normal', name = "Normal", desc = "Default wind speed" },
			-- { key = 'high', name = "High", desc = "High wind speed - for big teams" },
		-- },
	-- },

	-- {
		-- key  = 'tidal',
		-- name = 'Tidal Strength',
		-- desc = 'Tidal energy levels - How much energy is produced via tidal generators',
		-- type = 'list',
		-- section = 'Economy',
		-- def  = 'normal',
		-- items = {
			-- { key = 'low', name = "Low", desc = "Weak tidals - for smaller teams" },
			-- { key = 'normal', name = "Normal", desc = "Default tidals" },
			-- { key = 'high', name = "High", desc = "Strong tidals - for big teams" },
		-- },
	-- },
	
	-- {
		-- key     = 'extractorradius',
		-- name    = 'Extractor Radius',
		-- desc    = 'Sets the metal extractor radius',
		-- type    = 'number',
		-- section = 'Economy',
		-- def     = 100,
		-- min     = 1,
		-- max     = 10000,
		-- step    = 1,
	-- },
}

return options